@@ -257,7 +257,8 @@ gdk_offscreen_window_get_visual (GdkDrawable    *drawable)
 static void
 add_damage (GdkOffscreenWindow *offscreen,
 	    int x, int y,
-	    int w, int h)
+	    int w, int h,
+	    gboolean is_line)
 {
   GdkRectangle rect;
   GdkRegion *damage;
@@ -267,6 +268,26 @@ add_damage (GdkOffscreenWindow *offscreen,
   rect.width = w;
   rect.height = h;
 
+  if (is_line)
+    {
+      /* This should really take into account line width, line
+       * joins (and miter) and line caps. But these are hard
+       * to compute, rarely used and generally a pain. And in
+       * the end a snug damage rectangle is not that important
+       * as multiple damages are generally created anyway.
+       *
+       * So, we just add some padding around the rect.
+       * We use a padding of 3 pixels, plus an extra row
+       * below and on the right for the normal line size. I.E.
+       * line from (0,0) to (2,0) gets h=0 but is really
+       * at least one pixel tall.
+       */
+      rect.x -= 3;
+      rect.y -= 3;
+      rect.width += 7;
+      rect.height += 7;
+    }
+
   damage = gdk_region_rectangle (&rect);
   _gdk_window_add_damage (offscreen->wrapper, damage);
   gdk_region_destroy (damage);
@@ -300,7 +321,7 @@ gdk_offscreen_window_draw_drawable (GdkDrawable *drawable,
 		     xdest, ydest,
 		     width, height);
 
-  add_damage (offscreen, xdest, ydest, width, height);
+  add_damage (offscreen, xdest, ydest, width, height, FALSE);
 }
 
 static void
@@ -318,7 +339,7 @@ gdk_offscreen_window_draw_rectangle (GdkDrawable  *drawable,
   gdk_draw_rectangle (real_drawable,
 		      gc, filled, x, y, width, height);
 
-  add_damage (offscreen, x, y, width, height);
+  add_damage (offscreen, x, y, width, height, !filled);
 
 }
 
@@ -345,7 +366,7 @@ gdk_offscreen_window_draw_arc (GdkDrawable  *drawable,
 		height,
 		angle1,
 		angle2);
-  add_damage (offscreen, x, y, width, height);
+  add_damage (offscreen, x, y, width, height, !filled);
 }
 
 static void
@@ -381,7 +402,7 @@ gdk_offscreen_window_draw_polygon (GdkDrawable  *drawable,
 
 	add_damage (offscreen, min_x, min_y,
 		    max_x - min_x,
-		    max_y - min_y);
+		    max_y - min_y, !filled);
     }
 }
 
@@ -407,7 +428,7 @@ gdk_offscreen_window_draw_text (GdkDrawable  *drawable,
 		 text_length);
 
   /* Hard to compute the minimal size, not that often used anyway. */
-  add_damage (offscreen, 0, 0, private->width, private->height);
+  add_damage (offscreen, 0, 0, private->width, private->height, FALSE);
 }
 
 static void
@@ -432,7 +453,7 @@ gdk_offscreen_window_draw_text_wc (GdkDrawable	 *drawable,
 		    text_length);
 
   /* Hard to compute the minimal size, not that often used anyway. */
-  add_damage (offscreen, 0, 0, private->width, private->height);
+  add_damage (offscreen, 0, 0, private->width, private->height, FALSE);
 }
 
 static void
@@ -466,8 +487,9 @@ gdk_offscreen_window_draw_points (GdkDrawable  *drawable,
 	  }
 
 	add_damage (offscreen, min_x, min_y,
-		    max_x - min_x,
-		    max_y - min_y);
+		    max_x - min_x + 1,
+		    max_y - min_y + 1,
+		    FALSE);
     }
 }
 
@@ -492,7 +514,7 @@ gdk_offscreen_window_draw_segments (GdkDrawable  *drawable,
       min_x = max_x = segs[0].x1;
       min_y = max_y = segs[0].y1;
 
-	for (i = 1; i < nsegs; i++)
+	for (i = 0; i < nsegs; i++)
 	  {
 	    min_x = MIN (min_x, segs[i].x1);
 	    max_x = MAX (max_x, segs[i].x1);
@@ -506,7 +528,7 @@ gdk_offscreen_window_draw_segments (GdkDrawable  *drawable,
 
 	add_damage (offscreen, min_x, min_y,
 		    max_x - min_x,
-		    max_y - min_y);
+		    max_y - min_y, TRUE);
     }
 
 }
@@ -529,7 +551,7 @@ gdk_offscreen_window_draw_lines (GdkDrawable  *drawable,
   /* Hard to compute the minimal size, as we don't know the line
      width, and since joins are hard to calculate.
      Its not that often used anyway, damage it all */
-  add_damage (offscreen, 0, 0, private->width, private->height);
+  add_damage (offscreen, 0, 0, private->width, private->height, TRUE);
 }
 
 static void
@@ -556,7 +578,7 @@ gdk_offscreen_window_draw_image (GdkDrawable *drawable,
 		  width,
 		  height);
 
-  add_damage (offscreen, xdest, ydest, width, height);
+  add_damage (offscreen, xdest, ydest, width, height, FALSE);
 }
 
 
@@ -590,7 +612,7 @@ gdk_offscreen_window_draw_pixbuf (GdkDrawable *drawable,
 		   x_dither,
 		   y_dither);
 
-  add_damage (offscreen, dest_x, dest_y, width, height);
+  add_damage (offscreen, dest_x, dest_y, width, height, FALSE);
 
 }
 
@@ -601,7 +623,6 @@ _gdk_offscreen_window_new (GdkWindow     *window,
 			   GdkWindowAttr *attributes,
 			   gint           attributes_mask)
 {
-  GdkWindowObject *parent_private;
   GdkWindowObject *private;
   GdkOffscreenWindow *offscreen;
 
@@ -615,7 +636,6 @@ _gdk_offscreen_window_new (GdkWindow     *window,
   if (private->parent != NULL && GDK_WINDOW_DESTROYED (private->parent))
     return;
 
-  parent_private = (GdkWindowObject*) private->parent;
   private->impl = g_object_new (GDK_TYPE_OFFSCREEN_WINDOW, NULL);
   offscreen = GDK_OFFSCREEN_WINDOW (private->impl);
   offscreen->wrapper = window;
@@ -639,6 +659,7 @@ _gdk_offscreen_window_new (GdkWindow     *window,
 				      private->width,
 				      private->height,
 				      private->depth);
+  gdk_drawable_set_colormap (offscreen->pixmap, offscreen->colormap);
 }
 
 static gboolean
@@ -650,7 +671,6 @@ gdk_offscreen_window_reparent (GdkWindow *window,
   GdkWindowObject *private = (GdkWindowObject *)window;
   GdkWindowObject *new_parent_private = (GdkWindowObject *)new_parent;
   GdkWindowObject *old_parent;
-  GdkOffscreenWindow *offscreen;
   gboolean was_mapped;
 
   if (new_parent)
@@ -664,8 +684,6 @@ gdk_offscreen_window_reparent (GdkWindow *window,
 	return FALSE;
     }
 
-  offscreen = GDK_OFFSCREEN_WINDOW (private->impl);
-
   was_mapped = GDK_WINDOW_IS_MAPPED (window);
 
   gdk_window_hide (window);
@@ -1121,9 +1139,6 @@ gdk_offscreen_window_get_geometry (GdkWindow *window,
 				   gint      *depth)
 {
   GdkWindowObject *private = (GdkWindowObject *)window;
-  GdkOffscreenWindow *offscreen;
-
-  offscreen = GDK_OFFSCREEN_WINDOW (private->impl);
 
   g_return_if_fail (window == NULL || GDK_IS_WINDOW (window));
 
