@@ -24,6 +24,7 @@
 #include "gtkcellrendereraccel.h"
 #include "gtklabel.h"
 #include "gtkeventbox.h"
+#include "gtkmain.h"
 #include "gtkprivate.h"
 #include "gdk/gdkkeysyms.h"
 #include "gtkalias.h"
@@ -244,7 +245,16 @@ convert_keysym_state_to_string (GtkCellRendererAccel *accel,
   else 
     {
       if (accel->accel_mode == GTK_CELL_RENDERER_ACCEL_MODE_GTK)
-	return gtk_accelerator_get_label (keysym, mask);
+        {
+          if (!gtk_accelerator_valid (keysym, mask))
+            /* This label is displayed in a treeview cell displaying
+             * an accelerator key combination that is not valid according
+             * to gtk_accelerator_valid().
+             */
+            return g_strdup (C_("Accelerator", "Invalid"));
+
+          return gtk_accelerator_get_label (keysym, mask);
+        }
       else 
 	{
 	  gchar *name;
@@ -417,11 +427,14 @@ grab_key_callback (GtkWidget            *widget,
                                        event->group,
 				       NULL, NULL, NULL, &consumed_modifiers);
 
+  accel_mods = event->state;
+  gdk_keymap_add_virtual_modifiers (gdk_keymap_get_for_display (display), &accel_mods);
+
   accel_key = gdk_keyval_to_lower (event->keyval);
   if (accel_key == GDK_ISO_Left_Tab) 
     accel_key = GDK_Tab;
 
-  accel_mods = event->state & gtk_accelerator_get_default_mod_mask ();
+  accel_mods &= gtk_accelerator_get_default_mod_mask ();
 
   /* Filter consumed modifiers 
    */
