@@ -21,6 +21,7 @@
 #include "config.h"
 
 #include "gtkquartz.h"
+#include <gdk/quartz/gdkquartz.h>
 #include "gtkalias.h"
 
 NSImage *
@@ -83,7 +84,7 @@ target_to_pasteboard_type (const char *target)
     return [NSString stringWithUTF8String:target];
 }
 
-NSArray *
+NSSet *
 _gtk_quartz_target_list_to_pasteboard_types (GtkTargetList *target_list)
 {
   NSMutableSet *set = [[NSMutableSet alloc] init];
@@ -97,10 +98,10 @@ _gtk_quartz_target_list_to_pasteboard_types (GtkTargetList *target_list)
       g_free (target);
     }
 
-  return [set allObjects];
+  return set;
 }
 
-NSArray *
+NSSet *
 _gtk_quartz_target_entries_to_pasteboard_types (const GtkTargetEntry *targets,
 						guint                 n_targets)
 {
@@ -112,7 +113,7 @@ _gtk_quartz_target_entries_to_pasteboard_types (const GtkTargetEntry *targets,
       [set addObject:target_to_pasteboard_type (targets[i].target)];
     }
 
-  return [set allObjects];
+  return set;
 }
 
 GdkAtom 
@@ -159,7 +160,8 @@ _gtk_quartz_get_selection_data_from_pasteboard (NSPasteboard *pasteboard,
   selection_data = g_slice_new0 (GtkSelectionData);
   selection_data->selection = selection;
   selection_data->target = target;
-
+  if (!selection_data->display)
+    selection_data->display = gdk_display_get_default ();
   if (target == gdk_atom_intern_static_string ("UTF8_STRING"))
     {
       NSString *s = [pasteboard stringForType:NSStringPboardType];
@@ -260,7 +262,7 @@ _gtk_quartz_set_selection_data_for_pasteboard (NSPasteboard     *pasteboard,
   GdkDisplay *display;
   gint format;
   const guchar *data;
-  gint length;
+  NSUInteger length;
 
   target = gdk_atom_name (gtk_selection_data_get_target (selection_data));
   display = gtk_selection_data_get_display (selection_data);
@@ -316,8 +318,95 @@ _gtk_quartz_set_selection_data_for_pasteboard (NSPasteboard     *pasteboard,
       g_strfreev (list);
     }
   else
-    [pasteboard setData:[NSData dataWithBytesNoCopy:data
-	    	                             length:length
-			               freeWhenDone:NO]
-                forType:type];
+    [pasteboard setData:[NSData dataWithBytesNoCopy:(void *)data
+                                             length:length
+                                       freeWhenDone:NO]
+                                            forType:type];
 }
+
+/*
+ * Bundle-based functions for various directories. These almost work
+ * even when the application isn't in a bundle, becuase mainBundle
+ * paths point to the bin directory in that case. It's a simple matter
+ * to test for that and remove the last element.
+ */
+
+static gchar *
+get_bundle_path ()
+{
+  static gchar *path = NULL;
+  if (path == NULL)
+    {
+      gchar *base;
+      NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
+      gchar *resource_path = g_strdup ([[[NSBundle mainBundle] resourcePath] UTF8String]);
+      [pool drain];
+      base = g_path_get_basename (resource_path);
+      if (strcmp (base, "bin") == 0)
+	path = g_path_get_dirname (resource_path);
+      else
+	path = strdup (resource_path);
+      g_free (resource_path);
+      g_free (base);
+    }
+  return path;
+}
+
+const gchar *
+_gtk_get_datadir (void)
+{
+  static gchar *path = NULL;
+  if (path == NULL)
+    {
+      gchar *resource_dir = get_bundle_path ();
+      path = g_build_filename (resource_dir, "share", NULL);
+      g_free (resource_dir);
+    }
+  return path;
+}
+
+const gchar *
+_gtk_get_libdir (void)
+{
+  static gchar *path = NULL;
+  if (path == NULL)
+    {
+      gchar *resource_dir = get_bundle_path ();
+      path = g_build_filename (resource_dir, "lib", NULL);
+      g_free (resource_dir);
+    }
+  return path;
+}
+
+const gchar *
+_gtk_get_localedir (void)
+{
+  static gchar *path = NULL;
+  if (path == NULL)
+    {
+      gchar *resource_dir = get_bundle_path ();
+      path = g_build_filename (resource_dir, "share", "locale", NULL);
+      g_free (resource_dir);
+    }
+  return path;
+}
+
+const gchar *
+_gtk_get_sysconfdir (void)
+{
+  static gchar *path = NULL;
+  if (path == NULL)
+    {
+      gchar *resource_dir = get_bundle_path ();
+      path = g_build_filename (resource_dir, "etc", NULL);
+      g_free (resource_dir);
+    }
+  return path;
+}
+
+const gchar *
+_gtk_get_data_prefix (void)
+{
+  return get_bundle_path ();
+}
+
