@@ -282,13 +282,20 @@ gtk_status_icon_class_init (GtkStatusIconClass *class)
 							GDK_TYPE_SCREEN,
  							GTK_PARAM_READWRITE));
 
+  /**
+   * GtkStatusIcon:blinking:
+   *
+   * Whether or not the status icon is blinking.
+   *
+   * Deprecated: 2.22: This property will be removed in GTK+ 3
+   */
   g_object_class_install_property (gobject_class,
 				   PROP_BLINKING,
 				   g_param_spec_boolean ("blinking",
 							 P_("Blinking"),
 							 P_("Whether or not the status icon is blinking"),
 							 FALSE,
-							 GTK_PARAM_READWRITE));
+							 GTK_PARAM_READWRITE | G_PARAM_DEPRECATED));
 
   g_object_class_install_property (gobject_class,
 				   PROP_VISIBLE,
@@ -861,7 +868,7 @@ gtk_status_icon_init (GtkStatusIcon *status_icon)
   g_signal_connect_swapped (priv->tray_icon, "screen-changed",
 		    	    G_CALLBACK (gtk_status_icon_screen_changed), status_icon);
   priv->image = gtk_image_new ();
-  GTK_WIDGET_SET_FLAGS (priv->image, GTK_CAN_FOCUS);
+  gtk_widget_set_can_focus (priv->image, TRUE);
   gtk_container_add (GTK_CONTAINER (priv->tray_icon), priv->image);
   gtk_widget_show (priv->image);
 
@@ -900,6 +907,17 @@ gtk_status_icon_init (GtkStatusIcon *status_icon)
   priv->nid.uCallbackMessage = WM_GTK_TRAY_NOTIFICATION;
   priv->nid.uFlags = NIF_MESSAGE;
 
+  /* To help win7 identify the icon create it with an application "unique" tip */
+  if (g_get_prgname ())
+  {
+    WCHAR *wcs = g_utf8_to_utf16 (g_get_prgname (), -1, NULL, NULL, NULL);
+
+    priv->nid.uFlags |= NIF_TIP;
+    wcsncpy (priv->nid.szTip, wcs, G_N_ELEMENTS (priv->nid.szTip) - 1);
+    priv->nid.szTip[G_N_ELEMENTS (priv->nid.szTip) - 1] = 0;
+    g_free (wcs);
+  }
+
   if (!Shell_NotifyIconW (NIM_ADD, &priv->nid))
     {
       g_warning (G_STRLOC ": Shell_NotifyIcon(NIM_ADD) failed");
@@ -1732,16 +1750,16 @@ gtk_status_icon_button_press (GtkStatusIcon  *status_icon,
   if (handled)
     return TRUE;
 
-  if (event->button == 1 && event->type == GDK_BUTTON_PRESS)
-    {
-      emit_activate_signal (status_icon);
-      return TRUE;
-    }
-  else if (event->button == 3 && event->type == GDK_BUTTON_PRESS)
+  if (_gtk_button_event_triggers_context_menu (event))
     {
       emit_popup_menu_signal (status_icon, event->button, event->time);
       return TRUE;
     }
+  else if (event->button == 1 && event->type == GDK_BUTTON_PRESS)
+    {
+      emit_activate_signal (status_icon);
+      return TRUE;
+    }
 
   return FALSE;
 }
@@ -2019,7 +2037,8 @@ gtk_status_icon_get_storage_type (GtkStatusIcon *status_icon)
  * The caller of this function does not own a reference to the
  * returned pixbuf.
  * 
- * Return value: the displayed pixbuf, or %NULL if the image is empty.
+ * Return value: (transfer none): the displayed pixbuf,
+ *     or %NULL if the image is empty.
  *
  * Since: 2.10
  **/
@@ -2056,7 +2075,7 @@ gtk_status_icon_get_pixbuf (GtkStatusIcon *status_icon)
  *
  * Since: 2.10
  **/
-G_CONST_RETURN gchar *
+const gchar *
 gtk_status_icon_get_stock (GtkStatusIcon *status_icon)
 {
   GtkStatusIconPrivate *priv;
@@ -2088,7 +2107,7 @@ gtk_status_icon_get_stock (GtkStatusIcon *status_icon)
  *
  * Since: 2.10
  **/
-G_CONST_RETURN gchar *
+const gchar *
 gtk_status_icon_get_icon_name (GtkStatusIcon *status_icon)
 {
   GtkStatusIconPrivate *priv;
@@ -2118,7 +2137,7 @@ gtk_status_icon_get_icon_name (GtkStatusIcon *status_icon)
  *
  * If this function fails, @icon is left unchanged;
  *
- * Returns: the displayed icon, or %NULL if the image is empty
+ * Returns: (transfer none): the displayed icon, or %NULL if the image is empty
  *
  * Since: 2.14
  **/
@@ -2187,13 +2206,13 @@ gtk_status_icon_set_screen (GtkStatusIcon *status_icon,
 #endif
 }
 
-/** 
+/**
  * gtk_status_icon_get_screen:
  * @status_icon: a #GtkStatusIcon
  *
  * Returns the #GdkScreen associated with @status_icon.
  *
- * Return value: a #GdkScreen.
+ * Return value: (transfer none): a #GdkScreen.
  *
  * Since: 2.12
  */
@@ -2202,7 +2221,7 @@ gtk_status_icon_get_screen (GtkStatusIcon *status_icon)
 {
   g_return_val_if_fail (GTK_IS_STATUS_ICON (status_icon), NULL);
 
-#ifdef GDK_WINDOWING_X11   
+#ifdef GDK_WINDOWING_X11
   return gtk_window_get_screen (GTK_WINDOW (status_icon->priv->tray_icon));
 #else
   return gdk_screen_get_default ();
@@ -2298,7 +2317,7 @@ gtk_status_icon_set_visible (GtkStatusIcon *status_icon,
 #ifdef GDK_WINDOWING_X11
       if (visible)
 	gtk_widget_show (priv->tray_icon);
-      else if (GTK_WIDGET_REALIZED (priv->tray_icon)) 
+      else if (gtk_widget_get_realized (priv->tray_icon))
         {
 	  gtk_widget_hide (priv->tray_icon);
 	  gtk_widget_unrealize (priv->tray_icon);
@@ -2354,6 +2373,8 @@ gtk_status_icon_get_visible (GtkStatusIcon *status_icon)
  * this setting has no effect.
  *
  * Since: 2.10
+ *
+ * Deprecated: 2.22: This function will be removed in GTK+ 3
  **/
 void
 gtk_status_icon_set_blinking (GtkStatusIcon *status_icon,
@@ -2390,6 +2411,8 @@ gtk_status_icon_set_blinking (GtkStatusIcon *status_icon,
  * Return value: %TRUE if the icon is blinking
  *
  * Since: 2.10
+ *
+ * Deprecated: 2.22: This function will be removed in GTK+ 3
  **/
 gboolean
 gtk_status_icon_get_blinking (GtkStatusIcon *status_icon)
@@ -2791,7 +2814,7 @@ gtk_status_icon_get_tooltip_text (GtkStatusIcon *status_icon)
 /**
  * gtk_status_icon_set_tooltip_markup:
  * @status_icon: a #GtkStatusIcon
- * @markup: the contents of the tooltip for @status_icon, or %NULL
+ * @markup: (allow-none): the contents of the tooltip for @status_icon, or %NULL
  *
  * Sets @markup as the contents of the tooltip, which is marked up with
  *  the <link linkend="PangoMarkupFormat">Pango text markup language</link>.
@@ -2948,7 +2971,7 @@ gtk_status_icon_set_title (GtkStatusIcon *status_icon,
  *
  * Since: 2.18
  */
-G_CONST_RETURN gchar *
+const gchar *
 gtk_status_icon_get_title (GtkStatusIcon *status_icon)
 {
   GtkStatusIconPrivate *priv;
@@ -2969,5 +2992,44 @@ gtk_status_icon_get_title (GtkStatusIcon *status_icon)
 }
 
 
+/**
+ * gtk_status_icon_set_name:
+ * @status_icon: a #GtkStatusIcon
+ * @name: the name
+ *
+ * Sets the name of this tray icon.
+ * This should be a string identifying this icon. It is may be
+ * used for sorting the icons in the tray and will not be shown to
+ * the user.
+ *
+ * Since: 2.20
+ */
+void
+gtk_status_icon_set_name (GtkStatusIcon *status_icon,
+                          const gchar   *name)
+{
+  GtkStatusIconPrivate *priv;
+
+  g_return_if_fail (GTK_IS_STATUS_ICON (status_icon));
+
+  priv = status_icon->priv;
+
+#ifdef GDK_WINDOWING_X11
+  if (gtk_widget_get_realized (priv->tray_icon))
+    {
+      /* gtk_window_set_wmclass() only operates on non-realized windows,
+       * so temporarily unrealize the tray here
+       */
+      gtk_widget_hide (priv->tray_icon);
+      gtk_widget_unrealize (priv->tray_icon);
+      gtk_window_set_wmclass (GTK_WINDOW (priv->tray_icon), name, name);
+      gtk_widget_show (priv->tray_icon);
+    }
+  else
+    gtk_window_set_wmclass (GTK_WINDOW (priv->tray_icon), name, name);
+#endif
+}
+
+
 #define __GTK_STATUS_ICON_C__
 #include "gtkaliasdef.c"
